/*  PHONE  -  A program to demonstrate random updating by file pointer
 *  with a phone-directory type database.  Creates records and displays
 *  them based on their key fields, changes with conditional replace.
 */

#include <JER.H>
#include <SEARCH.H>

/*  Here are declarations for variables that are truly global, and should
 *  not have to be transferred from function to function as arguments.
 *  The first is a struct called PERSON which defines the record for the
 *  phone database file, then a FILE pointer to that file, itself.
 */
struct PERSON
{
    char name[30+1],
         phone[20 +1];
}
    record, compare;

FILE *database;
char filename[65];


/*  MAIN()  -  This main logic evaluates the command-line arguments, and
 *  invokes one of two other functions, depending on whether the arguments
 *  indicate that an addition to the file is required, or just a read
 *  from it.  The read function is designed to accomodate changes.  If
 *  the arguments are insufficient for either operation, then usage
 *  instructions are offered.
 */
main(argc,argv)
int argc;
char **argv;
{
    int  read_rec(),
         write_rec();

    if (argc < 3)
    {
        put("\n\nPHONE  Usage:  c:\> phone  fname lname number      (to add a record)");
        put("\n               c:\> phone  fname lname             (to locate one)\n\n");
        exit(0);
    }

/*  Start the filename for the phone data file with the path given in the
 *  environment variable PHONPATH.  If no such variable exists, then there
 *  can be a file called PHONBASE.DAT in every directory, and this program
 *  will use whichever one is current.  If the path is given, then all
 *  phone numbers will be kept in that one file, no matter where you are
 *  on the disk when you access the program.
 */
     if (getenv("PHONPATH"))
         strcpy(filename, getenv("PHONPATH"));
     strcat(filename,"PHONBASE.DAT");

/*  Copy the command-line first and last names to the name member of the
 *  comparison record, a temporary storage place.  Only the name is needed
 *  if the number of arguments is less than 4, as we will be doing a look-up.
 *  Otherwise, there is both a name and a phone number given, so we'll be
 *  doing a write/change, and the number needs copying, too.
 */
    strncpy(compare.name,argv[1],31);
    strncat(compare.name," ",31);
    strncat(compare.name,argv[2],31);

    if (argc < 4)
        read_rec();
    else                          /* name and number both,   mode = 1      */
    {
        strncpy(compare.phone, argv[3], 21);
        write_rec();
    }
    return(0);
}


/*  READ_REC()  -  Reading a record from the file is a straight-forward
 *  sequential file read in which records are read and their names compared
 *  with compare.name.  If there is a match, the reading loop is exited and
 *  the matching name and phone number displayed.  The file is opened for
 *  reading only, in binary mode.
 */
int read_rec()
{
    int foundit = FALSE;

    if ((database = fopen(filename,"rb")) == NULL)
    {
        put("\n\nPHONE:  Unable to open PHONBASE file to read...\n\n");
        exit(1);
    }

    while (! feof(database) && ! foundit)
    {
        fread(record.name, sizeof(record), 1, database);
        if (!stricmp(compare.name, record.name))
            foundit = TRUE;
    }
    fclose(database);

/*  foundit is set true only if a matching name was found in the file.
 *  If the search ended with no match, then just a message to that
 *  effect is printed.
 */
    if (foundit)
    {
        put("\nPHONE:      ");
        put(record.name);
        put("    ");
        put(record.phone);
        put("\n\n");
    }
    else
        put("\nPHONE:  Sorry, matching record not found...\n\n");

    return(NULL);
}


/*  WRITE_REC()  -  This is the most complex of the functions, as it
 *  includes coding to handle a change, as well as a simple replace.
 *  Since there are only two fields involved, and one of them is the
 *  key, there isn't a whole lot for a separate change function to do.
 *  This one simply checks before writing a new entry to the file, to
 *  see whether there's already a record with the same key.  If so, the
 *  user is prompted (y/n) whether to replace the record.  This provides
 *  a warning of any unintended replacement, and solves the problem of
 *  re-using records (it uses a re-write to change).
 */
int write_rec()
{
    int foundit = FALSE;
    char temp[3];                 /* buffer for a string of keyboard input */
    long offset;                  /* position in file to re-write a record */

/*  Important to check whether file can be opened to read, but this will
 *  fail the first time when data file doesn't exist.  If so, must open for
 *  write, then close, then open for read one more time before it can be
 *  checked for existing record key.  Again, mode is binary, so there's no
 *  problem with translation of CR and LF control codes.
 */
    if ((database = fopen(filename,"rb")) == NULL)
    {
        put("\n\nPHONE:  File PHONBASE doesn't exist, will create it...\n\n");
        if((database = fopen(filename,"wb"))== NULL)
        {
            put("\n\nPHONE:  Unable to open PHONBASE file to write...\n\n");
            exit(1);
        }
        fclose(database);
        if((database = fopen(filename,"rb")) == NULL)
            exit(1);
    }

/*  Once it's determined the file can be opened for read, then it is
 *  searched for existing key the same as what we're writing.  Before each
 *  read we must set a long index to the file pointer, so that record can
 *  be re-written according to its location in the file, if needed.  This
 *  is how a file update can be handled in C.
 */
    while (! feof(database) && ! foundit)
    {
        offset = ftell(database);
        fread(record.name, sizeof(record), 1, database);
        if (!strnicmp(compare.name, record.name, strlen(compare.name)))
            foundit = TRUE;
    }
    fclose(database);

/*  If the key field (the name) is found in the file already, then we need
 *  to find out whether it's a legitimate change or simply an error.  If
 *  the record exists, and a change is not wanted, we must exit.  If a
 *  change is wanted, then file must be closed and re-opened for write.
 *  Since it was opened for append, ALL writes are to the end of the file.
 *  Also notice that file pointer is moved to EOF before closing it, after
 *  the write, to prevent truncation.
 */
    if (foundit)
    {
        put("\nPHONE:  Name already exists - replace it (y/n)? ");
        input(temp,3);
        if (temp[0] == 'y' || temp[0] == 'Y')
        {
            database = fopen(filename,"wb");
            fseek(database, offset, SEEK_SET);
            fwrite(compare.name, sizeof(compare), 1, database);
            fseek(database, 0L, SEEK_END);
            fclose(database);
            put("\nPHONE:  Existing record replaced in PHONBASE file.\n");
        }
        else
        {
            put("\nPHONE:  Add-a-record cancelled at user request.\n");
            exit(0);
        }
    }

/*  However, if there is no matching record anyway, then just write the
 *  new record to the end of the file.  Nothing's to be gained by even
 *  sorting this file, because the file can be searched linearly in the
 *  time it would take to load an array of keys to it.  If we get to here,
 *  the file's still open for append, so the write goes to the end.
 */
    else
    {
        database = fopen(filename,"ab");
        fwrite(compare.name, sizeof(compare), 1, database);
        put("\nPHONE:  New record appended to PHONBASE file.\n");
        fclose(database);
    }
    return(NULL);
}


/*  PUT()  -  This is an implementation of my function that operates like
 *  puts(), except it does not append a newline to everything.  Thus,
 *  several strings can be output to the same line, such as a string
 *  constant, a string variable, and another string constant.  Newlines
 *  must be specified when wanted, such as with printf().
 */
int put(string)
char *string;
{
    int i=0;

    while (string[i])
        putchar(string[i++]);
    return(--i);
}


